@@ -5,9 +5,9 @@ from .single_stage import RotatedSingleStageDetector
 
 @ROTATED_DETECTORS.register_module()
 class RotatedRetinaNet(RotatedSingleStageDetector):
-    """Implementation of Rotated `RetinaNet.
+    """Implementation of Rotated `RetinaNet.`__
 
-    <https://arxiv.org/abs/1708.02002>`_
+    __ https://arxiv.org/abs/1708.02002
     """
 
     def __init__(self,
@@ -16,7 +16,8 @@ class RotatedRetinaNet(RotatedSingleStageDetector):
                  bbox_head,
                  train_cfg=None,
                  test_cfg=None,
-                 pretrained=None):
-        super(RotatedRetinaNet, self).__init__(backbone, neck, bbox_head,
-                                               train_cfg, test_cfg, pretrained)
-        self.bbox_head.init_weights()
+                 pretrained=None,
+                 init_cfg=None):
+        super(RotatedRetinaNet,
+              self).__init__(backbone, neck, bbox_head, train_cfg, test_cfg,
+                             pretrained, init_cfg)
